@@ -1,5 +1,5 @@
-/*  Copyright (c) 2014-2015 Andrzej <ndrwrdck@gmail.com>
- *  Copyright (c) 2015      Simon Steinbeiss <ochosi@xfce.org>
+/*  Copyright (c) 2014-2017 Andrzej <andrzejr@xfce.org>
+ *                2015      Simon Steinbeiss <ochosi@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -43,6 +43,7 @@
 #include "pulseaudio-plugin.h"
 #include "pulseaudio-config.h"
 #include "pulseaudio-menu.h"
+#include "pulseaudio-mpris.h"
 #include "pulseaudio-button.h"
 
 #define V_MUTED  0
@@ -80,6 +81,7 @@ struct _PulseaudioButton
 
   PulseaudioPlugin     *plugin;
   PulseaudioConfig     *config;
+  PulseaudioMpris      *mpris;
   PulseaudioVolume     *volume;
 
   GtkWidget            *image;
@@ -128,8 +130,8 @@ pulseaudio_button_init (PulseaudioButton *button)
   gtk_widget_set_can_focus(GTK_WIDGET(button), FALSE);
   gtk_widget_set_can_default (GTK_WIDGET (button), FALSE);
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-  gtk_button_set_use_underline (GTK_BUTTON (button),TRUE);
-  gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
+  gtk_button_set_use_underline (GTK_BUTTON (button), TRUE);
+  gtk_widget_set_focus_on_click (GTK_WIDGET (button), FALSE);
   gtk_widget_set_name (GTK_WIDGET (button), "pulseaudio-button");
 
   /* Preload icons */
@@ -160,10 +162,6 @@ pulseaudio_button_init (PulseaudioButton *button)
   gtk_widget_show (button->image);
 
   g_object_set (G_OBJECT (button), "has-tooltip", TRUE, NULL);
-
-  //button->align_box = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
-  //gtk_container_add (GTK_CONTAINER (button), button->align_box);
-  //gtk_widget_show (button->align_box);
 }
 
 
@@ -190,10 +188,10 @@ pulseaudio_button_button_press (GtkWidget      *widget,
 {
   PulseaudioButton *button = PULSEAUDIO_BUTTON (widget);
 
-  if(event->button == 1 && button->menu == NULL) /* left button */
+  if (event->button == 1 && button->menu == NULL) /* left button */
     {
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
-      button->menu = pulseaudio_menu_new (button->volume, button->config, widget);
+      button->menu = pulseaudio_menu_new (button->volume, button->config, button->mpris, widget);
       gtk_menu_attach_to_widget (GTK_MENU (button->menu), widget, NULL);
 
       if (button->deactivate_id == 0)
@@ -203,16 +201,18 @@ pulseaudio_button_button_press (GtkWidget      *widget,
              G_CALLBACK (pulseaudio_button_menu_deactivate), button);
         }
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
       gtk_menu_popup (GTK_MENU (button->menu),
                       NULL, NULL,
                       xfce_panel_plugin_position_menu, button->plugin,
-                      //NULL, NULL,
                       1,
                       event->time);
+G_GNUC_END_IGNORE_DEPRECATIONS
+
       return TRUE;
     }
 
-  if(event->button == 2) /* middle button */
+  if (event->button == 2) /* middle button */
     {
       pulseaudio_volume_toggle_muted (button->volume);
       return TRUE;
@@ -231,15 +231,14 @@ pulseaudio_button_scroll_event (GtkWidget *widget, GdkEventScroll *event)
   gdouble           new_volume;
 
 
-  if (event->direction == 1) // decrease volume
+  if (event->direction == 1)  // decrease volume
     new_volume = volume - volume_step;
-  else if (event->direction == 0) // increase volume
+  else if (event->direction == 0)  // increase volume
     new_volume = MIN (volume + volume_step, MAX (volume, 1.0));
   else
     new_volume = volume;
 
   pulseaudio_volume_set_volume (button->volume, new_volume);
-  //g_debug ("dir: %d %f -> %f", event->direction, volume, new_volume);
 
   return TRUE;
 }
@@ -364,12 +363,16 @@ pulseaudio_button_volume_changed (PulseaudioButton  *button,
 PulseaudioButton *
 pulseaudio_button_new (PulseaudioPlugin *plugin,
                        PulseaudioConfig *config,
+                       PulseaudioMpris  *mpris,
                        PulseaudioVolume *volume)
 {
   PulseaudioButton *button;
 
   g_return_val_if_fail (IS_PULSEAUDIO_PLUGIN (plugin), NULL);
   g_return_val_if_fail (IS_PULSEAUDIO_CONFIG (config), NULL);
+#ifdef HAVE_MPRIS2
+  g_return_val_if_fail (IS_PULSEAUDIO_MPRIS (mpris), NULL);
+#endif
   g_return_val_if_fail (IS_PULSEAUDIO_VOLUME (volume), NULL);
 
   button = g_object_new (TYPE_PULSEAUDIO_BUTTON, NULL);
@@ -377,6 +380,7 @@ pulseaudio_button_new (PulseaudioPlugin *plugin,
   button->plugin = plugin;
   button->volume = volume;
   button->config = config;
+  button->mpris = mpris;
   button->volume_changed_id =
     g_signal_connect_swapped (G_OBJECT (button->volume), "volume-changed",
                               G_CALLBACK (pulseaudio_button_volume_changed), button);
@@ -385,5 +389,3 @@ pulseaudio_button_new (PulseaudioPlugin *plugin,
 
   return button;
 }
-
-
