@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2014 Andrzej <ndrwrdck@gmail.com>
+ *  Copyright (C) 2014-2017 Andrzej <andrzejr@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -33,11 +33,10 @@
 #include <string.h>
 #endif
 
+#include <libxfce4panel/xfce-panel-plugin.h>
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
 #include <xfconf/xfconf.h>
-//#include <exo/exo.h>
-#include <libxfce4panel/xfce-panel-plugin.h>
 
 #include "pulseaudio-plugin.h"
 #include "pulseaudio-config.h"
@@ -46,9 +45,18 @@
 
 
 #define DEFAULT_ENABLE_KEYBOARD_SHORTCUTS         TRUE
+#define DEFAULT_SHOW_NOTIFICATIONS                TRUE
 #define DEFAULT_VOLUME_STEP                       6
 #define DEFAULT_VOLUME_MAX                        153
 
+#ifdef HAVE_MPRIS2
+#define DEFAULT_ENABLE_MPRIS                      TRUE
+#else
+#define DEFAULT_ENABLE_MPRIS                      FALSE
+#endif
+
+#define DEFAULT_MPRIS_PLAYERS                     ""
+
 
 
 static void                 pulseaudio_config_finalize       (GObject          *object);
@@ -73,9 +81,12 @@ struct _PulseaudioConfig
   GObject          __parent__;
 
   gboolean         enable_keyboard_shortcuts;
+  gboolean         show_notifications;
   guint            volume_step;
   guint            volume_max;
   gchar           *mixer_command;
+  gboolean         enable_mpris;
+  gchar           *mpris_players;
 };
 
 
@@ -84,9 +95,12 @@ enum
   {
     PROP_0,
     PROP_ENABLE_KEYBOARD_SHORTCUTS,
+    PROP_SHOW_NOTIFICATIONS,
     PROP_VOLUME_STEP,
     PROP_VOLUME_MAX,
     PROP_MIXER_COMMAND,
+    PROP_ENABLE_MPRIS,
+    PROP_MPRIS_PLAYERS,
     N_PROPERTIES,
   };
 
@@ -122,6 +136,15 @@ pulseaudio_config_class_init (PulseaudioConfigClass *klass)
 
 
 
+  g_object_class_install_property (gobject_class,
+                                   PROP_SHOW_NOTIFICATIONS,
+                                   g_param_spec_boolean ("show-notifications", NULL, NULL,
+                                                         DEFAULT_SHOW_NOTIFICATIONS,
+                                                         G_PARAM_READWRITE |
+                                                         G_PARAM_STATIC_STRINGS));
+
+
+
   g_object_class_install_property (gobject_class,
                                    PROP_VOLUME_STEP,
                                    g_param_spec_uint ("volume-step", NULL, NULL,
@@ -149,6 +172,26 @@ pulseaudio_config_class_init (PulseaudioConfigClass *klass)
                                                         G_PARAM_STATIC_STRINGS));
 
 
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_ENABLE_MPRIS,
+                                   g_param_spec_boolean ("enable-mpris", NULL, NULL,
+                                                         DEFAULT_ENABLE_MPRIS,
+                                                         G_PARAM_READWRITE |
+                                                         G_PARAM_STATIC_STRINGS));
+
+
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_MPRIS_PLAYERS,
+                                   g_param_spec_string ("mpris-players",
+                                                        NULL, NULL,
+                                                        DEFAULT_MPRIS_PLAYERS,
+                                                        G_PARAM_READWRITE |
+                                                        G_PARAM_STATIC_STRINGS));
+
+
+
   pulseaudio_config_signals[CONFIGURATION_CHANGED] =
     g_signal_new (g_intern_static_string ("configuration-changed"),
                   G_TYPE_FROM_CLASS (gobject_class),
@@ -164,9 +207,12 @@ static void
 pulseaudio_config_init (PulseaudioConfig *config)
 {
   config->enable_keyboard_shortcuts = DEFAULT_ENABLE_KEYBOARD_SHORTCUTS;
+  config->show_notifications        = DEFAULT_SHOW_NOTIFICATIONS;
   config->volume_step               = DEFAULT_VOLUME_STEP;
   config->volume_max                = DEFAULT_VOLUME_MAX;
   config->mixer_command             = g_strdup (DEFAULT_MIXER_COMMAND);
+  config->enable_mpris              = DEFAULT_ENABLE_MPRIS;
+  config->mpris_players             = g_strdup (DEFAULT_MPRIS_PLAYERS);
 }
 
 
@@ -198,6 +244,10 @@ pulseaudio_config_get_property (GObject    *object,
       g_value_set_boolean (value, config->enable_keyboard_shortcuts);
       break;
 
+    case PROP_SHOW_NOTIFICATIONS:
+      g_value_set_boolean (value, config->show_notifications);
+      break;
+
     case PROP_VOLUME_STEP:
       g_value_set_uint (value, config->volume_step);
       break;
@@ -210,6 +260,14 @@ pulseaudio_config_get_property (GObject    *object,
       g_value_set_string (value, config->mixer_command);
       break;
 
+    case PROP_ENABLE_MPRIS:
+      g_value_set_boolean (value, config->enable_mpris);
+      break;
+
+    case PROP_MPRIS_PLAYERS:
+      g_value_set_string (value, config->mpris_players);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -240,6 +298,16 @@ pulseaudio_config_set_property (GObject      *object,
         }
       break;
 
+    case PROP_SHOW_NOTIFICATIONS:
+      val_bool = g_value_get_boolean (value);
+      if (config->show_notifications != val_bool)
+        {
+          config->show_notifications = val_bool;
+          g_object_notify (G_OBJECT (config), "show-notifications");
+          g_signal_emit (G_OBJECT (config), pulseaudio_config_signals [CONFIGURATION_CHANGED], 0);
+        }
+      break;
+
     case PROP_VOLUME_STEP:
       val_uint = g_value_get_uint (value);
       if (config->volume_step != val_uint)
@@ -265,6 +333,23 @@ pulseaudio_config_set_property (GObject      *object,
       config->mixer_command = g_value_dup_string (value);
       break;
 
+    case PROP_ENABLE_MPRIS:
+      val_bool = g_value_get_boolean (value);
+      if (config->enable_mpris != val_bool)
+        {
+          config->enable_mpris = val_bool;
+          g_object_notify (G_OBJECT (config), "enable-mpris");
+          g_signal_emit (G_OBJECT (config), pulseaudio_config_signals [CONFIGURATION_CHANGED], 0);
+        }
+      break;
+
+    case PROP_MPRIS_PLAYERS:
+      g_free (config->mpris_players);
+      config->mpris_players = g_value_dup_string (value);
+      g_object_notify (G_OBJECT (config), "mpris-players");
+      g_signal_emit (G_OBJECT (config), pulseaudio_config_signals [CONFIGURATION_CHANGED], 0);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -285,6 +370,17 @@ pulseaudio_config_get_enable_keyboard_shortcuts (PulseaudioConfig *config)
 
 
 
+gboolean
+pulseaudio_config_get_show_notifications (PulseaudioConfig *config)
+{
+  g_return_val_if_fail (IS_PULSEAUDIO_CONFIG (config), DEFAULT_SHOW_NOTIFICATIONS);
+
+  return config->show_notifications;
+}
+
+
+
+
 guint
 pulseaudio_config_get_volume_step (PulseaudioConfig *config)
 {
@@ -318,6 +414,95 @@ pulseaudio_config_get_mixer_command (PulseaudioConfig *config)
 
 
 
+gboolean
+pulseaudio_config_get_enable_mpris (PulseaudioConfig *config)
+{
+  g_return_val_if_fail (IS_PULSEAUDIO_CONFIG (config), DEFAULT_ENABLE_MPRIS);
+
+  return config->enable_mpris;
+}
+
+
+
+
+gchar **
+pulseaudio_config_get_mpris_players (PulseaudioConfig *config)
+{
+  if (!IS_PULSEAUDIO_CONFIG (config))
+    {
+      return g_strsplit (DEFAULT_MPRIS_PLAYERS, ";", 1);
+    }
+
+  return g_strsplit (config->mpris_players, ";", 0);
+}
+
+static gint
+compare_players (gconstpointer item1, gconstpointer item2)
+{
+  return g_ascii_strcasecmp (item1, item2);
+}
+
+void
+pulseaudio_config_set_mpris_players (PulseaudioConfig  *config,
+                                     gchar            **players)
+{
+  GSList *player_array;
+  gchar  *player_string;
+  GValue  src = { 0, };
+  guint   index = 0;
+
+  g_return_if_fail (IS_PULSEAUDIO_CONFIG (config));
+
+  player_array = NULL;
+  for (guint i = 0; i < g_strv_length (players); i++) {
+    player_array = g_slist_prepend (player_array, players[i]);
+  }
+  player_array = g_slist_sort (player_array, (GCompareFunc) compare_players);
+
+  for (GSList *list = player_array; list != NULL; list = g_slist_next (list)) {
+    players[index] = list->data;
+    index++;
+  }
+
+  g_slist_free (player_array);
+
+  player_string = g_strjoinv (";", players);
+
+  g_value_init(&src, G_TYPE_STRING);
+  g_value_set_static_string(&src, player_string);
+
+  pulseaudio_config_set_property (G_OBJECT (config), PROP_MPRIS_PLAYERS, &src, NULL);
+}
+
+void
+pulseaudio_config_add_mpris_player (PulseaudioConfig *config,
+                                    gchar            *player)
+{
+  gchar **players;
+  gchar **player_list;
+  gchar  *players_string;
+  gchar  *player_string;
+
+  players = pulseaudio_config_get_mpris_players (config);
+  if (g_strv_contains ((const char * const *) players, player)) {
+      return;
+  }
+
+  players_string = g_strjoinv (";", players);
+  player_string = g_strjoin (";", players_string, player, NULL);
+  player_list = g_strsplit(player_string, ";", 0);
+
+  pulseaudio_config_set_mpris_players (config, player_list);
+
+  g_strfreev (player_list);
+  g_free (player_string);
+  g_free (players_string);
+  g_strfreev (players);
+}
+
+
+
+
 PulseaudioConfig *
 pulseaudio_config_new (const gchar     *property_base)
 {
@@ -335,6 +520,10 @@ pulseaudio_config_new (const gchar     *property_base)
       xfconf_g_property_bind (channel, property, G_TYPE_BOOLEAN, config, "enable-keyboard-shortcuts");
       g_free (property);
 
+      property = g_strconcat (property_base, "/show-notifications", NULL);
+      xfconf_g_property_bind (channel, property, G_TYPE_BOOLEAN, config, "show-notifications");
+      g_free (property);
+
       property = g_strconcat (property_base, "/volume-step", NULL);
       xfconf_g_property_bind (channel, property, G_TYPE_UINT, config, "volume-step");
       g_free (property);
@@ -347,6 +536,14 @@ pulseaudio_config_new (const gchar     *property_base)
       xfconf_g_property_bind (channel, property, G_TYPE_STRING, config, "mixer-command");
       g_free (property);
 
+      property = g_strconcat (property_base, "/enable-mpris", NULL);
+      xfconf_g_property_bind (channel, property, G_TYPE_BOOLEAN, config, "enable-mpris");
+      g_free (property);
+
+      property = g_strconcat (property_base, "/mpris-players", NULL);
+      xfconf_g_property_bind (channel, property, G_TYPE_STRING, config, "mpris-players");
+      g_free (property);
+
       g_object_notify (G_OBJECT (config), "enable-keyboard-shortcuts");
       g_signal_emit (G_OBJECT (config), pulseaudio_config_signals [CONFIGURATION_CHANGED], 0);
     }
